#include "extdll.h"
#include "util.h"

#include "cbase.h"
#include "player.h"
#include "trains.h"
#include "nodes.h"
#include "weapons.h"
#include "soundent.h"
#include "monsters.h"
#include "../engine/shake.h"
#include "decals.h"
#include "gamerules.h"


class CHook : public CBaseEntity 
{ 
void Spawn( void ); 
int Classify ( void );
void EXPORT HookTouch( CBaseEntity *pOther );
void KillHook( void );
void Think ( void );
Vector m_vPlayerHangOrigin; 
BOOL m_fPlayerAtEnd; 
short ropesprite; 
public: static CHook *HookCreate( void ); 
CBasePlayer *pevOwner;
void Precache( void ); 
};

LINK_ENTITY_TO_CLASS( grapple_hook, CHook ); 

CHook *CHook::HookCreate( void ) 
{ 
CHook *pHook = GetClassPtr( (CHook *)NULL ); 
pHook->pev->classname = MAKE_STRING("hook"); 
pHook->Spawn();
return pHook; 
}

void CHook::Spawn( ) 
{ 
pev->movetype = MOVETYPE_FLY; 
pev->solid = SOLID_BBOX; 
pev->gravity = 0.5; 
SET_MODEL(ENT(pev), "models/crossbow_bolt.mdl"); 
UTIL_SetOrigin( pev, pev->origin ); 
UTIL_SetSize(pev, Vector(0, 0, 0), Vector(0, 0, 0)); 
SetTouch( CHook::HookTouch ); 
pev->nextthink = gpGlobals->time + 0.2; 
} 

void CHook::Precache( ) 
{ 
PRECACHE_MODEL ("models/crossbow_bolt.mdl"); PRECACHE_SOUND("weapons/xbow_hitbod1.wav"); PRECACHE_SOUND("weapons/xbow_hitbod2.wav"); PRECACHE_SOUND("weapons/xbow_fly1.wav"); PRECACHE_SOUND("weapons/xbow_hit1.wav"); PRECACHE_SOUND("fvox/beep.wav"); 
ropesprite = PRECACHE_MODEL("sprites/steam1.spr"); 
} 

int CHook :: Classify ( void ) 
{ 
return CLASS_NONE; 
} 

void CHook::HookTouch( CBaseEntity *pOther ) 
{ 
SetTouch( NULL ); 
SetThink( NULL ); 
if (pOther->pev->takedamage) 
{ 
//TraceResult tr = UTIL_GetGlobalTrace( );
TraceResult tr;
    
	tr.fAllSolid		= gpGlobals->trace_allsolid;
	tr.fStartSolid		= gpGlobals->trace_startsolid;
	tr.fInOpen			= gpGlobals->trace_inopen;
	tr.fInWater			= gpGlobals->trace_inwater;
	tr.flFraction		= gpGlobals->trace_fraction;
	tr.flPlaneDist		= gpGlobals->trace_plane_dist;
	tr.pHit			= gpGlobals->trace_ent;
	
	tr.vecEndPos		= pevOwner->GetGunPosition( ) + gpGlobals->v_forward * 100;
	tr.vecPlaneNormal	= gpGlobals->trace_plane_normal;
	tr.iHitgroup		= gpGlobals->trace_hitgroup;
entvars_t *Owner; 
Owner = VARS( pev->owner );
pOther->TraceAttack(Owner, 5, pev->velocity.Normalize(), &tr, DMG_NEVERGIB );
ApplyMultiDamage( pev, Owner ); 
pev->velocity = Vector( 0, 0, 0 );
switch( RANDOM_LONG(0,1) ) 
{ 
case 0: EMIT_SOUND(ENT(pev), CHAN_WEAPON, "weapons/xbow_hitbod1.wav", 1, ATTN_NORM); 
break; 
case 1: EMIT_SOUND(ENT(pev), CHAN_WEAPON, "weapons/xbow_hitbod2.wav", 1, ATTN_NORM); 
break; 
} 


KillHook();
if ( !g_pGameRules->IsMultiplayer() ) 
{ 
Killed( pev, GIB_NEVER ); 
} 
return; 
} 
else { EMIT_SOUND_DYN(ENT(pev), CHAN_WEAPON, "weapons/xbow_hit1.wav", RANDOM_FLOAT(0.95, 1.0), ATTN_NORM, 0, 98 + RANDOM_LONG(0,7)); 
if ( FClassnameIs( pOther->pev, "worldspawn" ) ) 
{ 
Vector vecDir = pev->velocity.Normalize( ); 
UTIL_SetOrigin( pev, pev->origin - vecDir * 12 ); 
pev->angles = UTIL_VecToAngles( vecDir ); 
pev->solid = SOLID_NOT; 
pev->movetype = MOVETYPE_FLY; 
pev->velocity = Vector( 0, 0, 0 ); 
pev->avelocity.z = 0; 
pev->angles.z = RANDOM_LONG(0,360); 
SetThink( CHook::Think ); 
pev->nextthink = gpGlobals->time + 0.5; 
pevOwner->m_fHookInWall = TRUE; 
pevOwner->m_fActiveHook = TRUE; 
pevOwner->pev->movetype = MOVETYPE_FLY; 
} 
UTIL_Sparks( pev->origin );
}
}

void CHook::KillHook( void ) 
{ 
pevOwner->m_fHookButton = FALSE; 
pevOwner->pev->movetype = MOVETYPE_WALK; 
pevOwner->m_fActiveHook = FALSE; 
pevOwner->m_fHookInWall = FALSE; 
pevOwner->pev->gravity = 1; 
m_fPlayerAtEnd = FALSE; 
SUB_Remove(); 
}

void CHook::Think ( void ) 
{ 
if(!pevOwner->IsAlive( ) ) 
{ 
KillHook(); 
return; 
} 
if(!pevOwner->m_fHookButton)
{ 
KillHook(); 
return; 
} 

#define HOOK_PLAYER_PULL_SPEED 600

if(pevOwner->m_fHookInWall)
{ 
if(!m_fPlayerAtEnd) 
{ 
if(( pev->origin - pevOwner->pev->origin ).Length() >= 50.0) 
{ 
pevOwner->pev->velocity = (( pev->origin - pevOwner->pev->origin ) + pevOwner->m_vVecDirHookMove) * 10; 
pevOwner->pev->speed = HOOK_PLAYER_PULL_SPEED; 
} 
else 
{ 
m_vPlayerHangOrigin = pevOwner->pev->origin;
m_fPlayerAtEnd = TRUE;
}
}
if(m_fPlayerAtEnd)
{
pevOwner->pev->origin = m_vPlayerHangOrigin; 
pevOwner->pev->velocity = Vector(0, 0, 0); 
pevOwner->pev->gravity = 0; 
pevOwner->pev->speed = 0; 
}
} 
pev->nextthink = gpGlobals->time + 0.10;
} 

#define HOOK_SPEED 1000 

void CBasePlayer::FireHook ( void ) 
{
if(m_fActiveHook) 
TraceResult tr; 
m_iWeaponVolume = 300;
EMIT_SOUND_DYN(ENT(pev), CHAN_WEAPON, "weapons/xbow_fire1.wav", RANDOM_FLOAT(0.95, 1.0), ATTN_NORM, 0, 93 + RANDOM_LONG(0,0xF));
Vector anglesAim = pev->v_angle + pev->punchangle;
UTIL_MakeVectors( anglesAim ); 
anglesAim.x = -anglesAim.x; 
Vector vecSrc = GetGunPosition( ) - gpGlobals->v_up * 2; 
Vector vecDir = gpGlobals->v_forward;

CHook *pHook = CHook::HookCreate(); 
m_fActiveHook = TRUE; 
pHook->pev->origin = vecSrc; 
pHook->pev->angles = anglesAim;
pHook->pev->owner = edict(); 
pHook->pevOwner = this; 
pHook->pev->velocity = vecDir * HOOK_SPEED; 
pHook->pev->speed = HOOK_SPEED;
m_vVecDirHookMove = vecDir; 
pev->punchangle.x -= 2;
}