/***
*
*	WHL
*	Mine a la worms
*	(c est la armemine relookee)
*
****/
#include "extdll.h"
#include "util.h"
#include "cbase.h"
#include "monsters.h"
#include "weapons.h"
#include "nodes.h"
#include "player.h"
#include "gamerules.h"

#define TYPE_MINE	9
#define DEGATS_MINE	50
#define WEAPON_MINE	16
#define MINE_DEFAULT_GIVE	1
#define MINE_MAX_CARRY	5
#define MINE_WEIGHT	0


enum mine_e {
	MINE_IDLE1 = 0,
	MINE_FIDGET1,
	MINE_DRAW,
	MINE_DROP
};

class CMine : public CGrenade
{
	void Spawn( void );
	void Precache( void );

	void EXPORT MineSlide( CBaseEntity *pOther );
	void EXPORT MineThink( void );

// WHL ++
	float tps_MiseEnRoute;
	float tps_AvantBoom;
	float tps_ProchainBip;

	float CalcTemps (int type);
// WHL --

public:
	void Deactivate( void );
};
LINK_ENTITY_TO_CLASS( mine, CMine );

//=========================================================
// Deactivate - do whatever it is we do to an orphaned 
// mine when we don't want it in the world anymore.
//=========================================================
void CMine::Deactivate( void )
{
	pev->solid = SOLID_NOT;
	UTIL_Remove( this );
}


void CMine :: Spawn( void )
{
	Precache( );
	// motor
	pev->movetype = MOVETYPE_BOUNCE;
	pev->solid = SOLID_BBOX;

	SET_MODEL(ENT(pev), "models/w_satchel.mdl");
	UTIL_SetSize(pev, Vector( -4, -4, -4), Vector(4, 4, 4));
	UTIL_SetOrigin( pev, pev->origin );

	SetTouch( &CMine::MineSlide );
	SetUse( &CMine::DetonateUse );
	SetThink( &CMine::MineThink );

//WHL ++
	tps_MiseEnRoute = gpGlobals->time + 5.0; // 5 s pour fuir
	tps_AvantBoom = 0;
	tps_ProchainBip = 0;
//WHL --
	pev->nextthink = gpGlobals->time + 0.1;
    pev->health =20;
	pev->gravity = 0.5;
	pev->friction = 0.8;

	pev->dmg = DEGATS_MINE;
	// ResetSequenceInfo( );
	pev->sequence = 1;
}


void CMine::MineSlide( CBaseEntity *pOther )
{
	entvars_t	*pevOther = pOther->pev;

	// don't hit the guy that launched this grenade
	if ( pOther->edict() == pev->owner )
		return;

	// pev->avelocity = Vector (300, 300, 300);
	pev->gravity = 1;// normal gravity now

	// HACKHACK - On ground isn't always set, so look for ground underneath
	TraceResult tr;
	UTIL_TraceLine( pev->origin, pev->origin - Vector(0,0,10), ignore_monsters, edict(), &tr );

	if ( tr.flFraction < 1.0 )
	{
		// add a bit of static friction
		pev->velocity = pev->velocity * 0.95;
		pev->avelocity = pev->avelocity * 0.9;
		// play sliding sound, volume based on velocity
	}

	StudioFrameAdvance( );
}


void CMine :: MineThink( void )
{
//WHL ++
	CBaseEntity *pOther = NULL;
//WHL --
	StudioFrameAdvance( );
	pev->nextthink = gpGlobals->time + 0.1;

	if (!IsInWorld())
	{
		UTIL_Remove( this );
		return;
	}

	if (pev->waterlevel == 3)
	{
		pev->movetype = MOVETYPE_FLY;
		pev->velocity = pev->velocity * 0.8;
		pev->avelocity = pev->avelocity * 0.9;
		pev->velocity.z += 8;
	}
	else if (pev->waterlevel == 0)
	{
		pev->movetype = MOVETYPE_BOUNCE;
	}
	else
	{
		pev->velocity.z -= 8;
	}

// WHL ++
	if ((gpGlobals->time >= tps_MiseEnRoute) && (!tps_AvantBoom) )
	{
		if((pOther = UTIL_FindEntityInSphere( pOther, pev->origin , 64)) != NULL)
		{
			if(FClassnameIs(pOther->pev,"player"))
			{
				tps_AvantBoom = CalcTemps (TYPE_MINE);
				tps_ProchainBip = gpGlobals->time + 0.9;
			}
		}
	}
	if ( (tps_AvantBoom) && (gpGlobals->time >= tps_AvantBoom) )
	{
		SetThink( &CMine::Detonate );
	}
	if ( (tps_ProchainBip) && (gpGlobals->time >= tps_ProchainBip) )
	{
		EMIT_SOUND(ENT(pev), CHAN_VOICE, "misc/bip.wav", 1, ATTN_NORM);
		tps_ProchainBip = gpGlobals->time + 0.9;
	}
	if (pev->health <= 0)
	SetThink( &CMine::Detonate );
// WHL --
}

void CMine :: Precache( void )
{
	PRECACHE_MODEL("models/grenade.mdl");
	PRECACHE_SOUND("misc/bip.wav");
}


// WHL ++
float CMine::CalcTemps (int type)
{
	switch (type)
	{
	case 0:
		return gpGlobals->time;
		break;
	case 1:
		return gpGlobals->time + 1.0;
		break;
	case 2:
		return gpGlobals->time + 2.0;
		break;
	case 3:
		return gpGlobals->time + 3.0;
		break;
	case 9:
		return gpGlobals->time + (float) RANDOM_LONG( 0, 3 );
		break;
	}
}
// WHL --


// L arme qu on porte
//
//
//

class CArmeMine : public CBasePlayerWeapon
{
public:
	void Spawn( void );
	void Precache( void );
	int iItemSlot( void ) { return 5; }
	int GetItemInfo(ItemInfo *p);
	int AddToPlayer( CBasePlayer *pPlayer );
	void PrimaryAttack( void );
	BOOL CanDeploy( void );
	BOOL Deploy( void );
	BOOL IsUseable( void );
	
	void Holster( int skiplocal = 0 );
	void WeaponIdle( void );
	void Throw( void );
};
LINK_ENTITY_TO_CLASS( weapon_mine, CArmeMine ); //


//=========================================================
//=========================================================
int CArmeMine::AddToPlayer( CBasePlayer *pPlayer )
{
	int bResult = CBasePlayerItem::AddToPlayer( pPlayer );

	pPlayer->pev->weapons |= (1<<m_iId);

	if (bResult)
	{
		return AddWeapon( );
	}
	return FALSE;
}

void CArmeMine::Spawn( )
{
	Precache( );
	m_iId = WEAPON_MINE;
	SET_MODEL(ENT(pev), "models/w_satchel.mdl");

	m_iDefaultAmmo = MINE_DEFAULT_GIVE;
		
	FallInit();// get ready to fall down.
}


void CArmeMine::Precache( void )
{
	PRECACHE_MODEL("models/v_satchel.mdl");
	PRECACHE_MODEL("models/w_satchel.mdl");
	PRECACHE_MODEL("models/p_satchel.mdl");
	UTIL_PrecacheOther( "mine" );
}


int CArmeMine::GetItemInfo(ItemInfo *p)
{
	p->pszName = STRING(pev->classname);
	p->pszAmmo1 = "Mine";
	p->iMaxAmmo1 = MINE_MAX_CARRY;
	p->pszAmmo2 = NULL;
	p->iMaxAmmo2 = -1;
	p->iMaxClip = WEAPON_NOCLIP;
	p->iSlot = 0;
	p->iPosition = 5;
	p->iFlags = ITEM_FLAG_SELECTONEMPTY | ITEM_FLAG_LIMITINWORLD | ITEM_FLAG_EXHAUSTIBLE;
	p->iId = m_iId = WEAPON_MINE;
	p->iWeight = MINE_WEIGHT;

	return 1;
}

//=========================================================
//=========================================================
BOOL CArmeMine::IsUseable( void )
{
	if ( m_pPlayer->m_rgAmmo[ PrimaryAmmoIndex() ] > 0 ) 
	{
		// player is carrying some armemines
		return TRUE;
	}
	return FALSE;
}

BOOL CArmeMine::CanDeploy( void )
{
	if ( m_pPlayer->m_rgAmmo[ PrimaryAmmoIndex() ] > 0 ) 
	{
		// player is carrying some armemines
		return TRUE;
	}

	return FALSE;
}

BOOL CArmeMine::Deploy( )
{
	m_pPlayer->pev->viewmodel = MAKE_STRING("models/v_satchel.mdl");
	m_pPlayer->pev->weaponmodel = MAKE_STRING("models/p_satchel.mdl");
	SendWeaponAnim( MINE_DRAW );
	// use tripmine animations
	strcpy( m_pPlayer->m_szAnimExtention, "trip" );

	m_pPlayer->m_flNextAttack = UTIL_WeaponTimeBase() + 1.0;
	m_flTimeWeaponIdle = gpGlobals->time + RANDOM_FLOAT ( 10, 15 );
	return TRUE;
}


void CArmeMine::Holster( int skiplocal /* = 0 */ )
{
	m_pPlayer->m_flNextAttack = UTIL_WeaponTimeBase() + 0.5;
	
	SendWeaponAnim( MINE_DROP );

	EMIT_SOUND(ENT(m_pPlayer->pev), CHAN_WEAPON, "common/null.wav", 1.0, ATTN_NORM);

	if ( !m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType] )
	{
		m_pPlayer->pev->weapons &= ~(1<<WEAPON_MINE);
		SetThink( &CArmeMine::DestroyItem );
		pev->nextthink = gpGlobals->time + 0.1;
	}
}



void CArmeMine::PrimaryAttack()
{
	Throw( );
}

void CArmeMine::Throw( void )
{
	if (m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType])
	{
		Vector vecSrc = m_pPlayer->pev->origin;

		Vector vecThrow = gpGlobals->v_forward * 274 + m_pPlayer->pev->velocity;

		CBaseEntity *pArmeMine = Create( "mine", vecSrc, Vector( 0, 0, 0), m_pPlayer->edict() );
		pArmeMine->pev->velocity = vecThrow;
		pArmeMine->pev->avelocity.y = 400;

		SendWeaponAnim( MINE_DRAW );

		// player "shoot" animation
		m_pPlayer->SetAnimation( PLAYER_ATTACK1 );

		m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType]--;

		m_flNextPrimaryAttack = gpGlobals->time + 1.0;
	}
}


void CArmeMine::WeaponIdle( void )
{
	if ( !m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType] )
	{
		RetireWeapon();
		return;
	}

	if (m_flTimeWeaponIdle > gpGlobals->time)
		return;

	SendWeaponAnim( MINE_FIDGET1 );
	// use tripmine animations
	strcpy( m_pPlayer->m_szAnimExtention, "trip" );

	m_flTimeWeaponIdle = gpGlobals->time + RANDOM_FLOAT ( 10, 15 );// how long till we do this again.
}
