/***
*
*	Copyright (c) 2000, Black Mesa Security (BMS). All rights reserved.
*	
*	This product contains software technology licensed from Id 
*	Software, Inc. ("Id Technology").  Id Technology (c) 1996 Id Software, Inc. 
*	All Rights Reserved.
*
*   Use, distribution, and modification of this source code and/or resulting
*   object code is restricted to non-commercial enhancements to products from
*   BMS.  All other use, distribution, or modification is prohibited
*   without written permission from BMS.
*
****/
#if !defined( OEM_BUILD )

#include "extdll.h"
#include "util.h"
#include "cbase.h"
#include "monsters.h"
#include "weapons.h"
#include "nodes.h"
#include "player.h"
#include "gamerules.h"
#include "decals.h"
extern cvar_t tourlimit;

#define WEAPON_SL            18
int			   iSquidSpitSprite2;

enum sl_e {
	RPG_IDLE = 0,
	RPG_FIDGET,
	RPG_RELOAD_REACH,
	RPG_RELOAD_LOAD,		// to reload
	RPG_RELOAD_AIM,
	RPG_FIRE,		// to empty
	RPG_HOLSTER1,	// loaded
	RPG_DRAW1,		// loaded
	RPG_HOLSTER2,	// unloaded

	RPG_IDLE2,	// unloaded idle

};





class CSl : public CBasePlayerWeapon
{
public:
	int		Save( CSave &save );
	int		Restore( CRestore &restore );
	static	TYPEDESCRIPTION m_SaveData[];

	void Spawn( void );
	void Precache( void );
	void Reload( void );
	int iItemSlot( void ) { return 1; }
	int GetItemInfo(ItemInfo *p);
	int AddToPlayer( CBasePlayer *pPlayer );

	BOOL Deploy( void );

	void Holster( void );

	void PrimaryAttack( void );
	void SecondaryAttack( void );
	void WeaponIdle( void );

	
	BOOL ShouldWeaponIdle( void ) { return TRUE; };



	int m_cActiveRockets;// how many missiles in flight from this launcher right now?

};
LINK_ENTITY_TO_CLASS( weapon_sl, CSl );

TYPEDESCRIPTION	CSl::m_SaveData[] = 
{

	DEFINE_FIELD( CSl, m_cActiveRockets, FIELD_INTEGER ),
};
IMPLEMENT_SAVERESTORE( CSl, CBasePlayerWeapon );







class CSlRocket : public CGrenade
{
public:
	int		Save( CSave &save );
	int		Restore( CRestore &restore );
	static	TYPEDESCRIPTION m_SaveData[];
	void Spawn( void );
	void Precache( void );
	void EXPORT FollowThink( void );
	void EXPORT IgniteThink( void );
	void EXPORT RocketTouch( CBaseEntity *pEntity );



	static CSlRocket *CreateSlRocket( Vector vecOrigin, Vector vecAngles, CBaseEntity *pOwner, CSl *pLauncher );

	int m_iSpriteTexture;//initiallize the sprite for disc
	int m_iExplode;//initiallize the sprite for explosion
	int m_iGlow;//initiallize the sprite for afterglow


	int m_iTrail;
	float m_flIgniteTime;
	CSl *m_pLauncher;// pointer back to the launcher that fired me. 
};
LINK_ENTITY_TO_CLASS( sl_rocket, CSlRocket );

TYPEDESCRIPTION	CSlRocket::m_SaveData[] = 
{
	DEFINE_FIELD( CSlRocket, m_flIgniteTime, FIELD_TIME ),
	DEFINE_FIELD( CSlRocket, m_pLauncher, FIELD_CLASSPTR ),
};
IMPLEMENT_SAVERESTORE( CSlRocket, CGrenade );

//=========================================================
//=========================================================
CSlRocket *CSlRocket::CreateSlRocket( Vector vecOrigin, Vector vecAngles, CBaseEntity *pOwner, CSl *pLauncher )
{
	CSlRocket *pRocket = GetClassPtr( (CSlRocket *)NULL );

	UTIL_SetOrigin( pRocket->pev, vecOrigin );
	pRocket->pev->angles = vecAngles;
	pRocket->Spawn();
	pRocket->SetTouch( CSlRocket::RocketTouch );
	pRocket->m_pLauncher = pLauncher;// remember what RPG fired me. 
	pRocket->m_pLauncher->m_cActiveRockets++;// register this missile as active for the launcher
	pRocket->pev->owner = pOwner->edict();

	return pRocket;
}

//=========================================================
//=========================================================
void CSlRocket :: Spawn( void )
{
	Precache( );
	// motor
	pev->movetype = MOVETYPE_BOUNCE;
	pev->solid = SOLID_BBOX;

	SET_MODEL(ENT(pev), "models/spore.mdl");
	UTIL_SetSize(pev, Vector( 0, 0, 0), Vector(0, 0, 0));
	UTIL_SetOrigin( pev, pev->origin );

	pev->classname = MAKE_STRING("squidspit");

	SetThink( IgniteThink );
	SetTouch( ExplodeTouch );

	pev->angles.x -= 30;
	UTIL_MakeVectors( pev->angles );
	pev->angles.x = -(pev->angles.x + 30);

	pev->velocity = gpGlobals->v_forward * 250;
	pev->gravity = 0.5;

	pev->nextthink = gpGlobals->time + 0.4;

	pev->dmg = 2;
}

//=========================================================
//=========================================================
void CSlRocket :: RocketTouch ( CBaseEntity *pEntity )
{
		TraceResult tr;
	int		iPitch;

	// splat sound
	iPitch = RANDOM_FLOAT( 90, 110 );

		



	if ( !pEntity->pev->takedamage )
	{

		// make a splat on the wall
		UTIL_TraceLine( pev->origin, pev->origin + pev->velocity * 10, dont_ignore_monsters, ENT( pev ), &tr );
		UTIL_DecalTrace(&tr, DECAL_SPIT1 + RANDOM_LONG(0,1));

		// make some flecks
		MESSAGE_BEGIN( MSG_PVS, SVC_TEMPENTITY, tr.vecEndPos );
			WRITE_BYTE( TE_SPRITE_SPRAY );
			WRITE_COORD( tr.vecEndPos.x);	// pos
			WRITE_COORD( tr.vecEndPos.y);	
			WRITE_COORD( tr.vecEndPos.z);	
			WRITE_COORD( tr.vecPlaneNormal.x);	// dir
			WRITE_COORD( tr.vecPlaneNormal.y);	
			WRITE_COORD( tr.vecPlaneNormal.z);	
			WRITE_SHORT( iSquidSpitSprite2 );	// model
			WRITE_BYTE ( 5 );			// count
			WRITE_BYTE ( 30 );			// speed
			WRITE_BYTE ( 80 );			// noise ( client will divide by 100 )
		MESSAGE_END();
	}
	else
	{
		pEntity->TakeDamage ( pev, pev, 2, DMG_ACID );
	}

	SetThink ( SUB_Remove );
	pev->nextthink = gpGlobals->time;
}




//

//=========================================================
//=========================================================
void CSlRocket :: Precache( void )
{
	PRECACHE_MODEL("models/spore.mdl");
	m_iTrail = PRECACHE_MODEL("sprites/smoke.spr");
	m_iSpriteTexture = PRECACHE_MODEL( "sprites/white.spr" );//precache the sprite for disc
	m_iExplode = PRECACHE_MODEL( "sprites/fexplo.spr" );//precache the sprite for explosion
	m_iGlow = PRECACHE_MODEL( "sprites/hotglow.spr" );//precache the sprite for afterglow
	iSquidSpitSprite2 = PRECACHE_MODEL("sprites/tinyspit.spr");// client side spittle
	
}


void CSlRocket :: IgniteThink( void  )
{
	// pev->movetype = MOVETYPE_TOSS;

	pev->movetype = MOVETYPE_FLY;


	// make rocket sound






/*
	WRITE_BYTE( MSG_BROADCAST, SVC_TEMPENTITY );
	WRITE_BYTE( MSG_BROADCAST, TE_BEAMFOLLOW);
		WRITE_SHORT(entindex());	// entity
	WRITE_SHORT(MSG_BROADCAST, m_iTrail );	// model
	WRITE_BYTE( MSG_BROADCAST, 40 ); // life
	WRITE_BYTE( MSG_BROADCAST, 5 );  // width
	WRITE_BYTE( MSG_BROADCAST, 224 );   // r, g, b
	WRITE_BYTE( MSG_BROADCAST, 224 );   // r, g, b
	WRITE_BYTE( MSG_BROADCAST, 255 );   // r, g, b
	WRITE_BYTE( MSG_BROADCAST, 255 );	// brightness
*/
	m_flIgniteTime = gpGlobals->time;

	// set to follow laser spot
	SetThink( FollowThink );
	pev->nextthink = gpGlobals->time + 0.1;
}


void CSlRocket :: FollowThink( void  )
{
	CBaseEntity *pEntity = NULL;
	Vector vecTarget;
	Vector vecDir;
	float flDist, flMax, flDot;
	TraceResult tr;

	UTIL_MakeAimVectors( pev->angles );

	vecTarget = gpGlobals->v_forward;
	flMax = 4096;
	
	// Examine all entities within a reasonable radius
	while ((pEntity = UTIL_FindEntityByClassname( pEntity, "laser_spot" )) != NULL)
	{
		UTIL_TraceLine ( pev->origin, pEntity->pev->origin, dont_ignore_monsters, ENT(pev), &tr );
		// ALERT( at_console, "%f\n", tr.flFraction );
		if (tr.flFraction >= 0.90)
		{
			vecDir = pEntity->pev->origin - pev->origin;
			flDist = vecDir.Length( );
			vecDir = vecDir.Normalize( );
			flDot = DotProduct( gpGlobals->v_forward, vecDir );
			if ((flDot > 0) && (flDist * (1 - flDot) < flMax))
			{
				flMax = flDist * (1 - flDot);
				vecTarget = vecDir;
			}
		}
	}

	pev->angles = UTIL_VecToAngles( vecTarget );

	// this acceleration and turning math is totally wrong, but it seems to respond well so don't change it.
	float flSpeed = pev->velocity.Length();
	if (gpGlobals->time - m_flIgniteTime < 1.0)
	{
		pev->velocity = pev->velocity * 0.2 + vecTarget * (flSpeed * 0.8 + 400);
		if (pev->waterlevel == 3)
		{
			// go slow underwater
			if (pev->velocity.Length() > 300)
			{
				pev->velocity = pev->velocity.Normalize() * 300;
			}
			UTIL_BubbleTrail( pev->origin - pev->velocity * 0.1, pev->origin, 4 );
		} 
		else 
		{
			if (pev->velocity.Length() > 2000)
			{
				pev->velocity = pev->velocity.Normalize() * 2000;
			}
		}
	}
	else
	{
		if (pev->effects & EF_LIGHT)
		{
			pev->effects = 0;
			
		}
		pev->velocity = pev->velocity * 0.2 + vecTarget * flSpeed * 0.798;
		if (pev->waterlevel == 0 && pev->velocity.Length() < 1500)
		{
			Detonate( );
		}
	}
	// ALERT( at_console, "%.0f\n", flSpeed );

	pev->nextthink = gpGlobals->time + 0.1;
}








void CSl::Reload( void )
{
	int iResult;

	if ( m_iClip == 0 )
	{
		// don't bother with any of this if don't need to reload.
		return;
	}

	// because the RPG waits to autoreload when no missiles are active while  the LTD is on, the
	// weapons code is constantly calling into this function, but is often denied because 
	// a) missiles are in flight, but the LTD is on
	// or
	// b) player is totally out of ammo and has nothing to switch to, and should be allowed to
	//    shine the designator around
	//
	// Set the next attack time into the future so that WeaponIdle will get called more often
	// than reload, allowing the RPG LTD to be updated
	
	m_flNextPrimaryAttack = gpGlobals->time + 0.5;

	if ( m_cActiveRockets )
	{
		// no reloading when there are active missiles tracking the designator.
		// ward off future autoreload attempts by setting next attack time into the future for a bit. 
		return;
	}

	

	if (m_iClip == 0)
	{
		iResult = DefaultReload( 24, RPG_RELOAD_LOAD, 2 );
	}

	if (iResult)
	{
		m_flTimeWeaponIdle = gpGlobals->time + RANDOM_FLOAT ( 10, 15 );
	}
}

void CSl::Spawn( )
{
	Precache( );
	m_iId = WEAPON_SL;

	SET_MODEL(ENT(pev), "models/w_rpg.mdl");


	if ( g_pGameRules->IsMultiplayer() )
	{
		// more default ammo in multiplay. 
		m_iDefaultAmmo = RPG_DEFAULT_GIVE * 2;
	}
	else
	{
		m_iDefaultAmmo = RPG_DEFAULT_GIVE;
	}

	FallInit();// get ready to fall down.
}


void CSl::Precache( void )
{
PRECACHE_MODEL ("models/v_sporel.mdl");
	UTIL_PrecacheOther( "sl_rocket" );

	
}


int CSl::GetItemInfo(ItemInfo *p)
{
	p->pszName = STRING(pev->classname);
	p->pszAmmo1 = "sl";
	p->iMaxAmmo1 = 6;
	p->pszAmmo2 = NULL;
	p->iMaxAmmo2 = -1;
	p->iMaxClip = WEAPON_NOCLIP;
	p->iSlot = 0;
	p->iPosition = 2;
	p->iId = m_iId = WEAPON_SL;
	p->iFlags = 0;
	p->iWeight = RPG_WEIGHT;

	return 1;
}

int CSl::AddToPlayer( CBasePlayer *pPlayer )
{
	if ( CBasePlayerWeapon::AddToPlayer( pPlayer ) )
	{
		MESSAGE_BEGIN( MSG_ONE, gmsgWeapPickup, NULL, pPlayer->pev );
			WRITE_BYTE( m_iId );
		MESSAGE_END();
		return TRUE;
	}
	return FALSE;
}

BOOL CSl::Deploy( )
{
	
	if ( m_iClip == 0 )
	{
		return DefaultDeploy( "models/v_sporel.mdl", "models/p_rpg.mdl", RPG_DRAW1, "sl" );
	}

	return DefaultDeploy( "models/v_sporel.mdl", "models/p_rpg.mdl", RPG_DRAW1, "sl" );
}




void CSl::Holster( )
{
	m_fInReload = FALSE;// cancel any reload in progress.

	m_pPlayer->m_flNextAttack = gpGlobals->time + 0.5;
	// m_flTimeWeaponIdle = gpGlobals->time + RANDOM_FLOAT ( 10, 15 );
	SendWeaponAnim( RPG_HOLSTER1 );
	
}



void CSl::PrimaryAttack()
{ if (!m_pPlayer->m_iClientFindetour)
    return;
 if (!m_pPlayer->m_iClienttire)
	 return;
if (m_pPlayer->m_iClientarme == 10)
return;
if (m_iClip)
{m_pPlayer->m_iClientarme=5;

		m_pPlayer->m_iWeaponVolume = LOUD_GUN_VOLUME;
		m_pPlayer->m_iWeaponFlash = BRIGHT_GUN_FLASH;

		SendWeaponAnim( RPG_FIRE );

		// player "shoot" animation
		m_pPlayer->SetAnimation( PLAYER_ATTACK1 );

		UTIL_MakeVectors( m_pPlayer->pev->v_angle );
		Vector vecSrc = m_pPlayer->GetGunPosition( ) + gpGlobals->v_forward * 16 + gpGlobals->v_right * 8 + gpGlobals->v_up * -8;
		
		CSlRocket *pRocket = CSlRocket::CreateSlRocket( vecSrc, m_pPlayer->pev->v_angle, m_pPlayer, this );

		UTIL_MakeVectors( m_pPlayer->pev->v_angle );// RpgRocket::Create stomps on globals, so remake.
		pRocket->pev->velocity = pRocket->pev->velocity + gpGlobals->v_forward * DotProduct( m_pPlayer->pev->velocity, gpGlobals->v_forward );

		// firing RPG no longer turns on the designator. ALT fire is a toggle switch for the LTD.
		// Ken signed up for this as a global change (sjb)

		


		
	
		
		m_flNextPrimaryAttack = gpGlobals->time + 0.9;
		m_flTimeWeaponIdle = gpGlobals->time + 1.5;
	
	}
	else
	{
		PlayEmptySound( );
	}
}

void CSl::SecondaryAttack()
{

}


void CSl::WeaponIdle( void )
{


	ResetEmptySound( );

	if (m_flTimeWeaponIdle > gpGlobals->time)
		return;

	if (m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType])
	{
		int iAnim;
		float flRand = RANDOM_FLOAT(0, 1);

		{
			if ( m_iClip == 0 )
				iAnim = RPG_IDLE2;
			else
				iAnim = RPG_IDLE;

			m_flTimeWeaponIdle = gpGlobals->time + 90.0 / 15.0;
		}
		{
			if ( m_iClip == 0 )
				iAnim = RPG_FIDGET;
			else
				iAnim = RPG_FIDGET;

			m_flTimeWeaponIdle = gpGlobals->time + 3.0;
		}

		SendWeaponAnim( iAnim );
	}
	else
	{
		m_flTimeWeaponIdle = gpGlobals->time + 1;
	}
}






class CSlAmmo : public CBasePlayerAmmo
{
	void Spawn( void )
	{ 
		Precache( );
		SET_MODEL(ENT(pev), "models/w_rpgammo.mdl");
		CBasePlayerAmmo::Spawn( );
	}
	void Precache( void )
	{
		PRECACHE_MODEL ("models/w_rpgammo.mdl");
	
	}
	BOOL AddAmmo( CBaseEntity *pEntity ) 
	{ 
		int iGive;

		if ( g_pGameRules->IsMultiplayer() )
		{
			// hand out more ammo per rocket in multiplayer.
			iGive = AMMO_RPGCLIP_GIVE * 2;
		}
		else
		{
			iGive = AMMO_RPGCLIP_GIVE;
		}

		if (pEntity->GiveAmmo( iGive, "rockets", 6 ) != -1)
		{
		
			return TRUE;
		}
		return FALSE;
	}
};
LINK_ENTITY_TO_CLASS( ammo_slclip, CSlAmmo );

#endif